#include "../std.h"

#include "identity_gacl.h"
#include "permission_gacl.h"

#include "object_access_gacl.h"

extern "C" {
  extern GACLentry *GACLparseEntry(xmlNodePtr cur);
}

ObjectAccessGACL::ObjectAccessGACL(void) {
}

ObjectAccessGACL::ObjectAccessGACL(const ObjectAccess& o):ObjectAccess(o) {
}

ObjectAccessGACL::ObjectAccessGACL(const char* str) {
  GACLacl    *acl = NULL;
  GACLentry  *entry;
  {
    xmlDocPtr   doc;
    xmlNodePtr  cur;
    doc = xmlParseMemory(str,strlen(str));
    if (doc == NULL) return;
    cur = xmlDocGetRootElement(doc);
    if (xmlStrcmp(cur->name, (const xmlChar *) "gacl")) {
      free(doc);
      free(cur);
      return;
    };
    cur = cur->xmlChildrenNode;
    acl = GACLnewAcl();
    while (cur != NULL) {
      if(xmlNodeIsText(cur)) { cur=cur->next; continue; };
      entry = GACLparseEntry(cur);
      if (entry == NULL) {
        GACLfreeAcl(acl);
        xmlFreeDoc(doc);
        return;
      };
      GACLaddEntry(acl, entry);
      cur=cur->next;
    };
    xmlFreeDoc(doc);
  };
  if(acl == NULL) return;
  for(entry=acl->firstentry;entry;entry=(GACLentry*)(entry->next)) {
    GACLuser user; user.firstcred=entry->firstcred;
    IdentityGACL* id = new IdentityGACL(&user);
    if(!id) continue;
    PermissionGACL* perm = new PermissionGACL;
    if(!perm) { delete id; continue; };
    perm->allow(entry->allowed);
    perm->deny(entry->denied);
    use(id,perm);
  };
}

ObjectAccessGACL::ObjectAccessGACL(const GACLacl* acl) {
  GACLentry  *entry;
  for(entry=acl->firstentry;entry;entry=(GACLentry*)(entry->next)) {
    GACLuser user; user.firstcred=entry->firstcred;
    IdentityGACL* id = new IdentityGACL(&user);
    if(!id) continue;
    PermissionGACL* perm = new PermissionGACL;
    if(!perm) { delete id; continue; };
    perm->allow(entry->allowed);
    perm->deny(entry->denied);
    use(id,perm);
  };
}

ObjectAccessGACL::~ObjectAccessGACL(void) {
}

GACLacl* ObjectAccessGACL::get(void) {
  GACLacl    *acl = NULL;
  acl = GACLnewAcl();
  if(!acl) return NULL;
  for(int n = 0;;) {
    Item* item = (*this)[n];
    if(!item) break;
    Identity* id = item->id();
    Permission* perm = item->permission();
    if((!id) || (!perm)) continue;
    GACLuser* user = IdentityGACL(*id).get();
    if(!user) continue;
    GACLentry* entry = GACLnewEntry();
    if(!entry) { GACLfreeUser(user); continue; };
    GACLaddCred(entry,user->firstcred);
    user->firstcred=NULL; GACLfreeUser(user);
    GACLallowPerm(entry,(PermissionGACL(*perm)).allowed());
    GACLdenyPerm(entry,(PermissionGACL(*perm)).denied());
  };
  return acl;   
}

