#include "../std.h"
#include "../misc/log_time.h"
#include "environment.h"

std::string globus_loc("");
std::string globus_scripts_loc;
std::string nordugrid_loc("");
std::string nordugrid_bin_loc;
std::string nordugrid_libexec_loc;
std::string nordugrid_config_loc("");
std::string support_mail_address;
std::string globus_gridmap;
bool central_configuration = false;

bool file_exists(const char* name) {
  struct stat st;
  if(lstat(name,&st) != 0) return false;
  if(!S_ISREG(st.st_mode)) return false;
  return true;
}

bool read_env_vars(bool guess) {
  char* tmp;
  if(globus_loc.length() == 0) {
    tmp=getenv("GLOBUS_LOCATION");
    if((tmp == NULL) || (strlen(tmp) == 0)) {
      if(!guess) {
        olog<<"Error: GLOBUS_LOCATION environment variable not defined"<<std::endl;
        return false;
      }
      else {
        tmp="/opt/globus";
      };
    };
    globus_loc=tmp;
  };
  globus_scripts_loc=globus_loc+"/libexec";
  if(nordugrid_loc.length() == 0) {
    tmp=getenv("NORDUGRID_LOCATION");
    if((tmp == NULL) || (strlen(tmp) == 0)) {
      if(!guess) {
        olog<<"NORDUGRID_LOCATION environment variable is not defined"<<std::endl;
        return false;
      }
      else {
        tmp=DEFAULT_NORDUGRID_LOCATION;
      };
    };
    nordugrid_loc=tmp;
  };
  nordugrid_bin_loc=nordugrid_loc+"/bin";
  nordugrid_libexec_loc=nordugrid_loc+"/libexec/nordugrid";
  struct stat st;
  if((stat(nordugrid_libexec_loc.c_str(),&st) != 0) ||
     (!(S_ISDIR(st.st_mode)))) {
    nordugrid_libexec_loc=nordugrid_loc+"/libexec";
  };
  if(nordugrid_config_loc.length() == 0) {
    tmp=getenv("NORDUGRID_CONFIG");
    if((tmp == NULL) || (strlen(tmp) == 0)) {
      if(!central_configuration) {
        nordugrid_config_loc=nordugrid_loc+"/etc/grid-manager.conf";
        if(!file_exists(nordugrid_config_loc.c_str())) {
          nordugrid_config_loc="/etc/grid-manager.conf";
        };
        if(!file_exists(nordugrid_config_loc.c_str())) {
          olog<<"Configation file is missing at all guessed locations "<<nordugrid_loc<<"/etc/grid-manager.conf and /etc/grid-manager.conf"<<std::endl;
          olog<<"Use NORDUGRID_CONFIG variable for non-standard location"<<std::endl;
          return false;
        };
      } else {
        nordugrid_config_loc="/etc/nordugrid.conf";
        if(!file_exists(nordugrid_config_loc.c_str())) {
          nordugrid_config_loc="/etc/arc.conf";
        };
        if(!file_exists(nordugrid_config_loc.c_str())) {
          olog<<"Central configuration file is missing at guessed locations /etc/nordugrid.conf and /etc/arc.conf"<<std::endl;
          olog<<"Use NORDUGRID_CONFIG variable for non-standard location"<<std::endl;
          return false;
        };
      };
    } else {
      nordugrid_config_loc=tmp;
    };
  };
  setenv("NORDUGRID_CONFIG",nordugrid_config_loc.c_str(),1);
  if(support_mail_address.length() == 0) {
    char hn[100];
    support_mail_address="grid.manager@";
    if(gethostname(hn,99) == 0) {
      support_mail_address+=hn;
    }
    else {
      support_mail_address+="localhost";
    };
  };
  tmp=getenv("GRIDMAP");
  if((tmp == NULL) || (strlen(tmp) == 0)) {
    globus_gridmap="/etc/grid-security/grid-mapfile";
  }
  else { globus_gridmap=tmp; };
  return true;
}

