#include "../std.h"

#include "../misc/url_options.h"
#include "../misc/globus_error_utils.h"
#include "../misc/log_time.h"

#include <globus_gsi_credential.h>
#include <globus_gsi_proxy.h>

extern "C" {
#include <myproxy.h>
}

bool myproxy_renew_credential(const char* cred_path,const char* myproxy_url) {
  if(globus_module_activate(GLOBUS_GSI_PROXY_MODULE) != GLOBUS_SUCCESS) return false;

  URL url(myproxy_url);
  if(!url) {
    odlog(1)<<"Can't parse myproxy URL "<<myproxy_url<<endl;
    return false;
  };
  char* username;
  {
    globus_result_t r;
    globus_gsi_cred_handle_t h;
    r=globus_gsi_cred_handle_init(&h,GLOBUS_NULL);
    if(r != GLOBUS_SUCCESS) {
      odlog(1)<<GlobusResult(r)<<endl;
      return false;
    };
    r=globus_gsi_cred_read_proxy(h,cred_path);
    if(r != GLOBUS_SUCCESS) {
      odlog(1)<<GlobusResult(r)<<endl;
      globus_gsi_cred_handle_destroy(h);
      return false;
    };
    r=globus_gsi_cred_get_identity_name(h,&username);
    if(r != GLOBUS_SUCCESS) {
      odlog(1)<<GlobusResult(r)<<endl;
      globus_gsi_cred_handle_destroy(h);
      return false;
    };
    globus_gsi_cred_handle_destroy(h);
  };
  myproxy_socket_attrs_t* socket_attrs = 
              (myproxy_socket_attrs_t*)malloc(sizeof(myproxy_socket_attrs_t));
  myproxy_request_t* client_request = 
              (myproxy_request_t*)malloc(sizeof(myproxy_request_t));
  myproxy_response_t* server_response =
              (myproxy_response_t*)malloc(sizeof(myproxy_response_t));
  if(!socket_attrs || !client_request || !server_response) {
    free(socket_attrs); free(client_request); free(server_response);
    return false;
  };
  memset(socket_attrs,0,sizeof(myproxy_socket_attrs_t));
  memset(client_request,0,sizeof(myproxy_request_t));
  memset(server_response,0,sizeof(myproxy_response_t));
  char* outputfile;
  myproxy_set_delegation_defaults(socket_attrs,client_request);
  socket_attrs->pshost=strdup(url.Host().c_str());
  socket_attrs->psport=url.Port();
  client_request->authzcreds=strdup(cred_path);
  client_request->username=username;
  outputfile=(char*)cred_path;
  //client_request->proxy_lifetime=12*60*60;

  if(myproxy_init_client(socket_attrs) < 0) {
    odlog(1)<<"myproxy error: "<<verror_get_string()<<endl;
    goto failure;
  };
  if(myproxy_get_delegation(socket_attrs,client_request,server_response,
                                                           outputfile)!=0) {
    odlog(1)<<"myproxy error: "<<verror_get_string()<<endl;
    goto failure;
  };
  verror_clear();
  globus_module_deactivate(GLOBUS_GSI_PROXY_MODULE);
  myproxy_free(socket_attrs,client_request,server_response);
  return true;
failure:
  verror_clear();
  globus_module_deactivate(GLOBUS_GSI_PROXY_MODULE);
  myproxy_free(socket_attrs,client_request,server_response);
  return false;
}

